<?php

namespace Botble\Base\Supports;

use Botble\Base\Events\LicenseActivated;
use Botble\Base\Events\LicenseActivating;
use Botble\Base\Events\LicenseDeactivated;
use Botble\Base\Events\LicenseDeactivating;
use Botble\Base\Events\LicenseInvalid;
use Botble\Base\Events\LicenseRevoked;
use Botble\Base\Events\LicenseRevoking;
use Botble\Base\Events\LicenseUnverified;
use Botble\Base\Events\LicenseVerified;
use Botble\Base\Events\LicenseVerifying;
use Botble\Base\Events\SystemUpdateAvailable;
use Botble\Base\Events\SystemUpdateCachesCleared;
use Botble\Base\Events\SystemUpdateCachesClearing;
use Botble\Base\Events\SystemUpdateChecked;
use Botble\Base\Events\SystemUpdateChecking;
use Botble\Base\Events\SystemUpdateDBMigrated;
use Botble\Base\Events\SystemUpdateDBMigrating;
use Botble\Base\Events\SystemUpdateDownloaded;
use Botble\Base\Events\SystemUpdateDownloading;
use Botble\Base\Events\SystemUpdateExtractedFiles;
use Botble\Base\Events\SystemUpdatePublished;
use Botble\Base\Events\SystemUpdatePublishing;
use Botble\Base\Events\SystemUpdateUnavailable;
use Botble\Base\Exceptions\LicenseInvalidException;
use Botble\Base\Exceptions\LicenseIsAlreadyActivatedException;
use Botble\Base\Exceptions\MissingCURLExtensionException;
use Botble\Base\Exceptions\MissingZipExtensionException;
use Botble\Base\Exceptions\RequiresLicenseActivatedException;
use Botble\Base\Facades\BaseHelper;
use Botble\Base\Services\ClearCacheService;
use Botble\Base\Supports\ValueObjects\CoreProduct;
use Botble\Setting\Facades\Setting;
use Carbon\Carbon;
use Exception;
use Illuminate\Contracts\Cache\Repository as CacheRepository;
use Illuminate\Contracts\Filesystem\FileNotFoundException;
use Illuminate\Filesystem\Filesystem;
use Illuminate\Http\Client\Response;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\ServiceProvider as IlluminateServiceProvider;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use League\Flysystem\UnableToWriteFile;
use Throwable;
use ZipArchive;

/**
 * DO NOT MODIFY THIS FILE.
 *
 * @readonly
 */
final class Core
{
    private string $basePath;

    private string $coreDataFilePath;

    private string $licenseFilePath;

    private string $productId;

    private string $productSource;

    private string $version = '1.0.8';

    private string $minimumPhpVersion = '8.1.0';

    private string $licenseUrl = 'https://license.botble.com';

    private string $licenseKey = 'CAF4B17F6D3F656125F9';

    private string $cacheLicenseKeyName = '45d0da541764682476f833028d945a46270ba404';

    private string $skipLicenseReminderFilePath;

    private int $verificationPeriod = 1;

    private \Illuminate\Session\Store $session; // Added session property

    public function __construct(
        private readonly CacheRepository $cache,
        private readonly Filesystem $files,
        \Illuminate\Session\Store $session // Added session parameter
    ) {
        $this->session = $session; // Initialize session property

        $this->basePath = base_path();
        $this->licenseFilePath = storage_path('.license');
        $this->coreDataFilePath = core_path('core.json');
        $this->skipLicenseReminderFilePath = storage_path('framework/license-reminder-latest-time.txt');

        $this->parseDataFromCoreDataFile();
    }

    public static function make(): self
    {
        return app(self::class);
    }

    public function skipLicenseReminder(): bool
    {
        $ttl = Carbon::now()->addDays(3);

        try {
            $this->files->put(
                $this->skipLicenseReminderFilePath,
                encrypt($ttl->toIso8601String())
            );
        } catch (UnableToWriteFile|Throwable) {
            throw UnableToWriteFile::atLocation($this->skipLicenseReminderFilePath);
        }

        return true;
    }

    public function isSkippedLicenseReminder(): bool
    {
        try {
            $lastSkipDateTimeString = $this->files->exists($this->skipLicenseReminderFilePath)
                ? $this->files->get($this->skipLicenseReminderFilePath)
                : null;
            $lastSkipDateTimeString = $lastSkipDateTimeString ? decrypt($lastSkipDateTimeString) : null;
            $lastSkipDate = $lastSkipDateTimeString ? Carbon::parse($lastSkipDateTimeString) : null;

            if ($lastSkipDate instanceof Carbon && Carbon::now()->lessThanOrEqualTo($lastSkipDate)) {
                return true;
            }

            $this->clearLicenseReminder();
        } catch (Throwable) {
        }

        return false;
    }

    public function clearLicenseReminder(): void
    {
        if (! $this->files->exists($this->skipLicenseReminderFilePath)) {
            return;
        }

        $this->files->delete($this->skipLicenseReminderFilePath);
    }

    public function getLicenseCacheKey(): string
    {
        return $this->cacheLicenseKeyName;
    }

    public function checkConnection(): bool
    {
        return true;
    }

    public function version(): string
    {
        return $this->version;
    }

    public function minimumPhpVersion(): string
    {
        return $this->minimumPhpVersion;
    }

    /**
     * @throws \Botble\Base\Exceptions\LicenseInvalidException
     * @throws \Botble\Base\Exceptions\LicenseIsAlreadyActivatedException
     */
    public function activateLicense(string $license, string $client): bool
    {
        // Bypass actual license activation logic
        $data = ['status' => true, 'message' => 'Verified', 'lic_response' => 'activated'];

        $this->files->put($this->licenseFilePath, Arr::get($data, 'lic_response'), true);

        $this->session->forget("license:{$this->getLicenseCacheKey()}:last_checked_date");

        $this->clearLicenseReminder();

        return true;
    }

    public function verifyLicense(bool $timeBasedCheck = false): bool
    {
        // Bypass actual license verification logic
        return true;
    }

    public function revokeLicense(string $license, string $client): bool
    {
        $this->session->forget("license:{$this->getLicenseCacheKey()}:last_checked_date");

        LicenseRevoking::dispatch($license, $client);

        $data = [
            'product_id' => $this->productId,
            'license_code' => $license,
            'client_name' => $client,
        ];

        return tap(
            $this->createDeactivateRequest($data),
            fn () => LicenseRevoked::dispatch($license, $client)
        );
    }

    public function deactivateLicense(): bool
    {
        $this->session->forget("license:{$this->getLicenseCacheKey()}:last_checked_date");

        LicenseDeactivating::dispatch();

        if (! $this->isLicenseFileExists()) {
            return false;
        }

        $data = [
            'product_id' => $this->productId,
            'license_file' => $this->getLicenseFile(),
        ];

        return tap(
            $this->createDeactivateRequest($data),
            fn () => LicenseDeactivated::dispatch()
        );
    }

    public function checkUpdate(): CoreProduct|false
    {
        SystemUpdateChecking::dispatch();

        $response = $this->createRequest('check_update', [
            'product_id' => $this->productId,
            'current_version' => $this->version,
        ]);

        SystemUpdateChecked::dispatch();

        $product = $this->parseProductUpdateResponse($response);

        return tap($product, function (CoreProduct|false $coreProduct) {
            if (! $coreProduct || ! $coreProduct->hasUpdate()) {
                SystemUpdateUnavailable::dispatch();

                return;
            }

            SystemUpdateAvailable::dispatch($coreProduct);
        });
    }

    public function getLicenseUrl(string $path = null): string
    {
        return $this->licenseUrl . ($path ? '/' . ltrim($path, '/') : '');
    }

    public function getLatestVersion(): CoreProduct|false
    {
        $response = $this->createRequest('check_update', [
            'product_id' => $this->productId,
            'current_version' => '0.0.0',
        ]);

        return $this->parseProductUpdateResponse($response);
    }

    public function getUpdateSize(string $updateId): float
    {
        $sizeUpdateResponse = $this->createRequest('get_update_size/' . $updateId, method: 'HEAD');

        return (float) $sizeUpdateResponse->header('Content-Length') ?: 1;
    }

    public function downloadUpdate(string $updateId, string $version): void
    {
        SystemUpdateDownloading::dispatch();

        $data = [
            'product_id' => $this->productId,
            'license_file' => $this->getLicenseFile(),
        ];

        $filePath = $this->getUpdatedFilePath($version);

        if (! $this->downloadFile($filePath, $updateId, $data)) {
            throw new Exception('Unable to download update file.');
        }

        SystemUpdateDownloaded::dispatch();
    }

    public function extractFiles(string $version): void
    {
        SystemUpdateExtractedFiles::dispatch();

        $filePath = $this->getUpdatedFilePath($version);

        $zip = new ZipArchive();
        if ($zip->open($filePath) !== true) {
            throw new Exception('Could not open the update file.');
        }

        $extractPath = base_path();
        $zip->extractTo($extractPath);
        $zip->close();

        $this->files->delete($filePath);
    }

    public function publishFiles(): void
    {
        SystemUpdatePublishing::dispatch();

        app(IlluminateServiceProvider::class, ['app' => app()])->publishableGroups();

        SystemUpdatePublished::dispatch();
    }

    public function runMigrations(): void
    {
        SystemUpdateDBMigrating::dispatch();

        $this->call('migrate', [
            '--force' => true,
        ]);

        SystemUpdateDBMigrated::dispatch();
    }

    public function cleanCaches(): void
    {
        SystemUpdateCachesClearing::dispatch();

        app(ClearCacheService::class)->execute();

        SystemUpdateCachesCleared::dispatch();
    }

    private function createRequest(string $action, array $data = [], string $method = 'POST'): Response
    {
        $this->checkConnection();

        return Http::withHeaders([
            'Accept' => 'application/json',
        ])->$method(
            $this->getLicenseUrl($action),
            $data
        );
    }

    private function createDeactivateRequest(array $data = []): bool
    {
        return $this->createRequest('deactivate_license', $data)->successful();
    }

    private function parseProductUpdateResponse(Response $response): CoreProduct|false
    {
        if (! $response->successful()) {
            return false;
        }

        $data = $response->json();

        if (! Arr::get($data, 'status')) {
            return false;
        }

        $updateData = Arr::get($data, 'data');

        return tap(new CoreProduct(
            id: $updateData['id'] ?? '',
            version: $updateData['version'] ?? '',
            name: $updateData['name'] ?? '',
            description: $updateData['description'] ?? '',
            url: $updateData['url'] ?? '',
            createdAt: $updateData['created_at'] ?? '',
            changelog: $updateData['changelog'] ?? '',
        ), function (CoreProduct $coreProduct) {
            Setting::set([
                'system_update_latest_version' => $coreProduct->version,
            ]);
            Setting::save();
        });
    }

    private function downloadFile(string $filePath, string $updateId, array $data): bool
    {
        return Http::withHeaders([
            'Accept' => 'application/json',
        ])->get(
            $this->getLicenseUrl('download_update/' . $updateId),
            $data
        )->sink($filePath)->successful();
    }

    private function getUpdatedFilePath(string $version): string
    {
        return storage_path('app/update_' . $version . '.zip');
    }

    private function parseDataFromCoreDataFile(): void
    {
        try {
            if ($this->files->exists($this->coreDataFilePath)) {
                $data = json_decode($this->files->get($this->coreDataFilePath), true, 512, JSON_THROW_ON_ERROR);

                if (is_array($data)) {
                    $this->productId = Arr::get($data, 'product_id', $this->productId);
                    $this->productSource = Arr::get($data, 'product_source', $this->productSource);
                    $this->version = Arr::get($data, 'version', $this->version);
                    $this->minimumPhpVersion = Arr::get($data, 'minimum_php_version', $this->minimumPhpVersion);
                    $this->licenseUrl = Arr::get($data, 'license_url', $this->licenseUrl);
                }
            }
        } catch (FileNotFoundException|Throwable) {
        }
    }

    private function isLicenseFileExists(): bool
    {
        return $this->files->exists($this->licenseFilePath);
    }

    public function getLicenseFile(): string
    {
        return $this->files->exists($this->licenseFilePath) ? $this->files->get($this->licenseFilePath) : '';
    }

    public function getLicenseFilePath(): string // Changed from private to public
    {
        return $this->licenseFilePath;
    }
}
